#!/usr/bin/env bash

cd "$(dirname "$0")"

# turn on verbose debugging output for parabuild logs.
exec 4>&1; export BASH_XTRACEFD=4; set -x
# make errors fatal
set -e
# complain about unreferenced environment variables
set -u

shopt -s globstar

if [ -z "$AUTOBUILD" ] ; then
    exit 1
fi

if [ "$OSTYPE" = "cygwin" ] ; then
    autobuild="$(cygpath -u $AUTOBUILD)"
else
    autobuild="$AUTOBUILD"
fi

top="$(pwd)"
stage="$top/stage"

# load autobuild provided shell functions and variables
source_environment_tempfile="$stage/source_environment.sh"
"$autobuild" source_environment > "$source_environment_tempfile"
. "$source_environment_tempfile"

CRASHPAD_SOURCE_DIR="crashpad"

version_str="8e85e6dd"

build=${AUTOBUILD_BUILD_ID:=0}
echo "${version_str}.${build}" > "${stage}/VERSION.txt"

case "$AUTOBUILD_PLATFORM" in
    windows*)
        # Fetch the library source
        cmd.exe /c gclient config --name crashpad https://git.alchemyviewer.org/alchemy/thirdparty/crashpad/crashpad.git
        cmd.exe /c gclient sync -r alchemy
        cmd.exe /c gclient runhooks
    ;;
    *)
        # Fetch the library source
        gclient config --name crashpad https://git.alchemyviewer.org/alchemy/thirdparty/crashpad/crashpad.git
        gclient sync -r alchemy
    ;;
esac

pushd "$CRASHPAD_SOURCE_DIR"
    case "$AUTOBUILD_PLATFORM" in
        windows*)
            load_vsvars

            mkdir -p "$stage/bin/debug"
            mkdir -p "$stage/bin/release"
            mkdir -p "$stage/lib/debug"
            mkdir -p "$stage/lib/release"
            mkdir -p "$stage/include/crashpad/client"
            mkdir -p "$stage/include/crashpad/util"
            mkdir -p "$stage/include/crashpad/base"

            if [ "$AUTOBUILD_ADDRSIZE" = 32 ]
            then
                targetcpu="x86"
            else
                targetcpu="x64"
            fi
            cmd /c $(cygpath -aw "../buildtools/win/gn.exe") gen out/Debug --args="is_debug=true dynamic_crt=true target_cpu=\"${targetcpu}\""
            ninja -C out/Debug

            cp -a out/Debug/crashpad_handler.com "$stage/bin/debug"
            cp -a out/Debug/crashpad_handler.exe "$stage/bin/debug"
            cp -a out/Debug/crashpad_handler.pdb "$stage/bin/debug"
            cp -a out/Debug/obj/client/client.lib "$stage/lib/debug"
            cp -a out/Debug/obj/util/util.lib "$stage/lib/debug"
            cp -a out/Debug/obj/third_party/mini_chromium/mini_chromium/base/base.lib "$stage/lib/debug"

            cmd /c $(cygpath -aw "../buildtools/win/gn.exe") gen out/Release --args="is_debug=false dynamic_crt=true target_cpu=\"${targetcpu}\""
            ninja -C out/Release

            cp -a out/Release/crashpad_handler.com "$stage/bin/release"
            cp -a out/Release/crashpad_handler.exe "$stage/bin/release"
            cp -a out/Release/crashpad_handler.pdb "$stage/bin/release"
            cp -a out/Release/obj/client/client.lib "$stage/lib/release"
            cp -a out/Release/obj/util/util.lib "$stage/lib/release"
            cp -a out/Release/obj/third_party/mini_chromium/mini_chromium/base/base.lib "$stage/lib/release"

            pushd "client"
            cp -a **/*.h --parents "$stage/include/crashpad/client"
            popd
            pushd "util"
                cp -a **/*.h --parents "$stage/include/crashpad/util"
            popd
            pushd "third_party/mini_chromium/mini_chromium/base"
                cp -a **/*.h --parents "$stage/include/crashpad/base"
            popd
        ;;

        darwin*)
            gn gen out/Debug --args=is_debug=true
            ninja -C out/Debug

            gn gen out/Release --args=is_debug=false
            ninja -C out/Release
        ;;

        linux*)
            # Default target per AUTOBUILD_ADDRSIZE
            opts="${TARGET_OPTS:--m$AUTOBUILD_ADDRSIZE -fPIC -DPIC}"

            # Handle any deliberate platform targeting
            if [ -z "${TARGET_CPPFLAGS:-}" ]; then
                # Remove sysroot contamination from build environment
                unset CPPFLAGS
            else
                # Incorporate special pre-processing flags
                export CPPFLAGS="${TARGET_CPPFLAGS:-}"
            fi
        ;;
    esac
    mkdir -p "$stage/LICENSES"
    cp -a LICENSE "$stage/LICENSES/crashpad.txt"
popd
